from flask import Blueprint, request, jsonify
from werkzeug.exceptions import NotFound, BadRequest

from flaschengeist.modules import Plugin
from flaschengeist.system.decorator import login_required
from flaschengeist.system.controller import roleController

roles_bp = Blueprint("roles", __name__)
_permission_edit = "roles_edit"
_permission_delete = "roles_delete"


class RolesPlugin(Plugin):
    def __init__(self, config):
        super().__init__(config, roles_bp, permissions=[_permission_edit, _permission_delete])


######################################################
#                       Routes                       #
#                                                    #
# /roles              POST: register new             #
#                      GET: get all roles            #
# /roles/permissions   GET: get all permissions      #
# /roles/<rid>         GET: get role with rid        #
#                      PUT: modify role / permission #
#                   DELETE: remove role              #
######################################################


@roles_bp.route("/roles", methods=["POST"])
@login_required(permission=_permission_edit)
def add_role(**kwargs):
    data = request.get_json()
    if not data or "name" not in data:
        raise BadRequest
    if "permissions" in data:
        permissions = data["permissions"]
    role = roleController.create_role(data["name"], permissions)
    return jsonify({"ok": "ok", "id": role.id})


@roles_bp.route("/roles", methods=["GET"])
@login_required()
def list_roles(**kwargs):
    roles = roleController.get_all()
    return jsonify(roles)


@roles_bp.route("/roles/permissions", methods=["GET"])
@login_required()
def list_permissions(**kwargs):
    permissions = roleController.get_permissions()
    return jsonify(permissions)


@roles_bp.route("/roles/<rid>", methods=["GET"])
@login_required()
def __get_role(rid, **kwargs):
    role = roleController.get(rid)
    if role:
        return jsonify({"id": role.id, "name": role, "permissions": role.permissions})
    raise NotFound


@roles_bp.route("/roles/<rid>", methods=["PUT"])
@login_required(permission=_permission_edit)
def __edit_role(rid, **kwargs):
    role = roleController.get(rid)

    data = request.get_json()
    if "name" in data:
        role.name = data["name"]
    if "permissions" in data:
        roleController.set_permissions(role, data["permissions"])
    roleController.update_role(role)
    return jsonify({"ok": "ok"})


@roles_bp.route("/roles/<rid>", methods=["DELETE"])
@login_required(permission=_permission_delete)
def __delete_role(rid, **kwargs):
    role = roleController.get(rid)
    roleController.delete(role)

    return jsonify({"ok": "ok"})
